body {
    padding: 3px 1em 1em 1em;
    margin: 0;
}

div#diazo-debug {
    height: 14em;
    overflow: scroll;
    background-color: rgba(255, 255, 255, 0.8);
}

section {
    padding: 3px;
}

pre.runtrace span.node, pre.runtrace span.closing, pre.runtrace span.comment { background: #CCC; }
pre.runtrace span.node.match { background: #AFA; }
pre.runtrace span.node.no-match { background: #FAA; }
pre.runtrace span.attr.match { background: #5F5; }
pre.runtrace span.attr.no-match { background: #F55; }
